'use client';

import { createContext, useContext, useState, useEffect, ReactNode, useCallback } from 'react';
import { User } from 'firebase/auth';
import { AdminUser } from '@/types/education';
import { onAdminAuthStateChanged, getAdminData } from '@/utils/admin_auth';

interface AdminContextType {
  user: User | null;
  isAdmin: boolean;
  adminData: AdminUser | null;
  loading: boolean;
  error: string | null;
  refreshAdminData: () => Promise<void>;
}

const AdminContext = createContext<AdminContextType>({
  user: null,
  isAdmin: false,
  adminData: null,
  loading: true,
  error: null,
  refreshAdminData: async () => {},
});

export function useAdmin() {
  return useContext(AdminContext);
}

interface AdminProviderProps {
  children: ReactNode;
}

export function AdminProvider({ children }: AdminProviderProps) {
  const [user, setUser] = useState<User | null>(null);
  const [isAdmin, setIsAdmin] = useState(false);
  const [adminData, setAdminData] = useState<AdminUser | null>(null);
  const [loading, setLoading] = useState(true);
  const [error, setError] = useState<string | null>(null);

  // Cache admin data in localStorage
  const cacheAdminData = useCallback((data: AdminUser | null) => {
    if (data) {
      localStorage.setItem('adminData', JSON.stringify({
        data,
        timestamp: Date.now(),
      }));
    } else {
      localStorage.removeItem('adminData');
    }
  }, []);

  // Get cached admin data
  const getCachedAdminData = useCallback(() => {
    const cached = localStorage.getItem('adminData');
    if (!cached) return null;

    const { data, timestamp } = JSON.parse(cached);
    // Cache expires after 5 minutes
    if (Date.now() - timestamp > 5 * 60 * 1000) {
      localStorage.removeItem('adminData');
      return null;
    }

    return data;
  }, []);

  // Fetch admin data with caching
  const fetchAdminData = useCallback(async (user: User) => {
    try {
      setError(null);
      const data = await getAdminData(user);
      setAdminData(data);
      cacheAdminData(data);
      return data;
    } catch (error) {
      console.error('Error fetching admin data:', error);
      setError('Failed to load admin data. Please try again.');
      const cachedData = getCachedAdminData();
      if (cachedData) {
        console.log('Using cached admin data');
        setAdminData(cachedData);
        return cachedData;
      }
      setAdminData(null);
      return null;
    }
  }, [cacheAdminData, getCachedAdminData]);

  // Refresh admin data
  const refreshAdminData = useCallback(async () => {
    if (user && isAdmin) {
      setLoading(true);
      await fetchAdminData(user);
      setLoading(false);
    }
  }, [user, isAdmin, fetchAdminData]);

  useEffect(() => {
    // Try to load cached data immediately
    const cachedData = getCachedAdminData();
    if (cachedData) {
      console.log('Using cached admin data while loading');
      setAdminData(cachedData);
    }

    const unsubscribe = onAdminAuthStateChanged(async (user, isAdmin) => {
      console.log('Auth State Changed:', { 
        userExists: !!user, 
        userEmail: user?.email,
        isAdmin 
      });
      
      setUser(user);
      setIsAdmin(isAdmin);
      
      if (user && isAdmin) {
        await fetchAdminData(user);
      } else {
        setAdminData(null);
        cacheAdminData(null);
      }
      
      setLoading(false);
    });

    return () => {
      unsubscribe();
    };
  }, [cacheAdminData, getCachedAdminData, fetchAdminData]);

  return (
    <AdminContext.Provider 
      value={{ 
        user, 
        isAdmin, 
        adminData, 
        loading, 
        error,
        refreshAdminData 
      }}
    >
      {children}
    </AdminContext.Provider>
  );
} 